/****************************************************************************/
/*                                                                          */
/*  Copyrighted by PostModern Computing Technologies, Inc                   */
/*                                                                          */
/*  Copyright 1993, 1994 by PostModern Computing Technologies, Inc.         */
/*                          1897 Landings Drive                             */
/*                          Mountain View, California, 94043, USA           */
/*                          Telephone (415) 967-6169                        */
/*                          Facsimile (415) 967-6212                        */
/*                                                                          */
/*  This code is the sole property of PostModern Computing Technologies,    */
/*  Inc.  and is  protected  by  copyright under the laws of the United     */
/*  States.   This  program is confidential, proprietary, and  a  trade     */
/*  secret,  not  to  be disclosed  without written authorization  from     */
/*  PostModern Computing Technologies, Inc.  Any use,  duplication,  or     */
/*  disclosure  of  this  program  by other than  PostModern  Computing     */
/*  Technologies, Inc. and its assigned licensees is strictly forbidden     */
/*  by law.                                                                 */
/*                                                                          */
/*  All rights reserved.                                                    */
/*                                                                          */
/****************************************************************************/

#include "irbrowser.h"

// -----------------------------
// Static initialization
// -----------------------------

CORBA_IR::Repository*                                              IRBrowser::repository = NULL;
char                                                              IRBrowser::command_buffer[100];
CORBA_IR::Container::IDLSequence(CORBA_IR__Contained)*              IRBrowser::contentsResult;
CORBA_IR::Container::IDLSequence(CORBA_IR__Contained)*              IRBrowser::newContentsResult;
CORBA_IR::Container::IDLSequence(CORBA_IR__Container__Description)* IRBrowser::descriptionResult;
int                                                               IRBrowser::arg = -1;

void IRBrowser::printHelp()
{
  cout << "\n  Command summary: (Try 'contents' to get started.  Or ";
  cout << "\n                    type 'info [command]' for details)";
  cout << "\n                                            ";
  cout << "\n         Object Type                 Command";
  cout << "\n        --------------              --------";
  cout << "\n                                            ";
  cout << "\n         (Repository)              lookup_id";
  cout << "\n                                            ";
  cout << "\n         (Contained)                  within";
  cout << "\n                                    describe";
  cout << "\n                                            ";  
  cout << "\n         (Container)                contents";
  cout << "\n                                 lookup_name";
  cout << "\n                           describe_contents";
  cout << "\n                                            ";  
  cout << "\n         (InterfaceDef)   describe_interface";
  cout << "\n                             base_interfaces";
  cout << "\n                                            ";
  cout << "\n         (Arbitrary)                    name";
  cout << "\n                                          id";
  cout << "\n                                  defined_in";
  cout << "\n                                            ";
  cout << "\n         (Some)                         type";
  cout << "\n                                       value";
  cout << "\n                                      result";
  cout << "\n                                        mode";
  cout << "\n                                            ";  
  cout << "\n         (Admin)                        quit";
  cout << "\n                                        show";
  cout << "\n                                        info";
  cout << "\n                                        help";  
}

void IRBrowser::printPrompt()
{
  cout << "\n Browse > ";
  cout.flush();
}

void IRBrowser::getInput()
{
  cin >> command_buffer;
}


void IRBrowser::lookup_id()
{
  if (arg != -1)
    {
      cout << "\nOperation lookup_id takes no argument!";
      return;
    }

  cout << "\n Specify search_id > ";
       
  char  search_id[500];
  cin >> search_id;

  CORBA_IR::RepositoryId searchId(search_id);
      
  CORBA_IR::Contained* contained = repository->lookup_id(searchId);

  if (contained == NULL) { cout << "\nFailed"; return; }
  else                  
   {
    if (newContentsResult)
     {
      for (CORBA::ULong j = newContentsResult->length(); j > 0 ; j--) 
       {
	 newContentsResult->remove(j-1);
       }
     }
    else
     {
       newContentsResult 
	= new CORBA_IR::Container::IDLSequence(CORBA_IR__Contained);
     }

    newContentsResult->append(contained);
   }
 
  update(); 

  prettyPrintContentsResult();
  
}

void IRBrowser::within()
{
  // The user must supply an argument so that we know which Contained
  // object from the last query result is meant.

    if (arg == -1)
    {
      cout << "\nYou must supply a #i argument for this operation.";
      return;
    }

    if ((contentsResult == NULL) || (arg >= contentsResult->length())
				 || (arg < 0))
	{
	  cout << "Argument range error.";
	  cout.flush();
	  return;
	}

    CORBA_IR::Contained * contained = (*contentsResult)[arg];

    CORBA_IR::Contained::IDLSequence(CORBA_IR__Container)* result = contained->within();

   // Because anything derived from Container is also derived from Contained
   // we can convert 'result' to be the new 'contentsResult'.  First remove
   // everything from the old contentsResult.

  releaseContentsResult();
   
  for (CORBA::ULong i=0; i < result->length(); i++) 
    {
       CORBA_IR::Contained* element = CORBA_IR::Contained::_narrow((*result)[i]);

       contentsResult->append(element);
    }

    prettyPrintContentsResult();

}

void IRBrowser::update()
{
  if (newContentsResult) 
     {
        releaseContentsResult();
        if (contentsResult == NULL)
            contentsResult = new CORBA_IR::Container::IDLSequence(CORBA_IR__Contained);

	for (CORBA::ULong i = 0; i < newContentsResult->length(); i++ ) 
	{

	   contentsResult->append((*newContentsResult)[i]);
	}

	for (CORBA::ULong j = newContentsResult->length(); j > 0 ; j--) 
	{
	   newContentsResult->remove(j-1);
	}
     }
}

void IRBrowser::releaseContentsResult()
{
   if (contentsResult) 
     {
	for (CORBA::ULong i = contentsResult->length(); i > 0 ; i--) 
	{
	   contentsResult->remove(i-1);
	}
     }
}

const char * IRBrowser::interfaceType(CORBA_IR::Contained* contained)
{

  CORBA_IR::ModuleDef *moduleDef = CORBA_IR::ModuleDef::_narrow(contained);
  if (moduleDef != NULL)          return "      (ModuleDef)";

  CORBA_IR::InterfaceDef *interfaceDef = CORBA_IR::InterfaceDef::_narrow(contained);
  if (interfaceDef != NULL)       return "   (InterfaceDef)";


  CORBA_IR::AttributeDef *attributeDef = CORBA_IR::AttributeDef::_narrow(contained);
  if (attributeDef != NULL)       return "   (AttributeDef)";


  CORBA_IR::OperationDef *operationDef = CORBA_IR::OperationDef::_narrow(contained);
  if (operationDef != NULL)       return "   (OperationDef)";


  CORBA_IR::ParameterDef *parameterDef = CORBA_IR::ParameterDef::_narrow(contained);
  if (parameterDef != NULL)       return "   (ParameterDef)";


  CORBA_IR::TypeDef *typeDef = CORBA_IR::TypeDef::_narrow(contained);
  if (typeDef != NULL)            return "        (TypeDef)";


  CORBA_IR::ConstantDef *constantDef = CORBA_IR::ConstantDef::_narrow(contained);
  if (constantDef != NULL)        return "    (ConstantDef)";

  CORBA_IR::ExceptionDef *exceptionDef = CORBA_IR::ExceptionDef::_narrow(contained);
  if (exceptionDef != NULL)       return "   (ExceptionDef)";

                                return "   --- Unknown?!";

}



void IRBrowser::describe()
{
  // The user must supply an argument so that we know which Contained
  // object is meant.

    if (arg == -1)
    {
      cout << "\nYou must supply a #i argument for this operation.";
      return;
    }

    if ((contentsResult == NULL) || (arg >= contentsResult->length())
				 || (arg < 0))
	{
	  cout << "Argument range error.";
	  cout.flush();
	  return;
	}

    CORBA_IR::Contained * contained = (*contentsResult)[arg];

    CORBA_IR::Contained::Description* result = contained->describe();

    cout << *result;
}



void IRBrowser::contents()
{
  // If the user has not specified an argument, apply contents() to
  // the Repository itself.  If the user has supplied an argument,
  // _attempt_ to apply contents() to the last computed contentsResult.
  // Since the result of the last computed contentsResult is a sequence
  // of Contained objects (and not Container), a call to this function
  // with an argument will succeed only if we can _narrow the corresponding
  // element in the last contentsResult to something derived from Container,
  // i.e., InterfaceDef or ModuleDef.  We call contentsHelper if the user
  // has specified an argument; otherwise this function handles it.


  if (arg != -1)
    {
      contentsHelper();
      return;
    }      
 
  cout << "\n Specify limit_type > ";

  char desired_interface[500];
  cin >> desired_interface;

  CORBA_IR::InterfaceName limit_type(desired_interface);

  CORBA::Boolean exclude_inherited = CORBA::TRUE;
  char exclude_char[10];
  cout << "\n Specify exclude_inherited ? (y) " ;
  cin >> exclude_char;
  if ( exclude_char[0] == 'n' || exclude_char[0] == 'N' )
 	exclude_inherited = CORBA::FALSE;

  newContentsResult = repository->contents(limit_type, exclude_inherited);
  update();

  CORBA::Long i;

  if (contentsResult == NULL)
    {
      cout << "\n(Empty result)\n";
      return;
    }
  
  cout << "\n";
  for (i = 0; i < contentsResult->length(); i++)
    {
      cout << "# "
	   << i
	   << interfaceType((*contentsResult)[i])
	   << "     "
	   << *((*contentsResult)[i]->name());
      cout << "\n";
    }
}

void IRBrowser::contentsHelper()
{

    if ((contentsResult == NULL) || (arg >= contentsResult->length())
				 || (arg < 0))
	{
	  cout << "Argument range error.";
	  cout.flush();
	  return;
	}

  
  CORBA_IR::Contained * contained = (*contentsResult)[arg];
  if (contained == NULL)
    {
      cout << "\nError.";
      return;
    }

  cout << "\n Specify limit_type > ";
       
  char desired_interface[500];
  cin >> desired_interface;

  CORBA::Boolean exclude_inherited = CORBA::TRUE;
  char exclude_char[10];
  cout << "\n Specify exclude_inherited ? (y) " ;
  cin >> exclude_char;
  if ( exclude_char[0] == 'n' || exclude_char[0] == 'N' )
 	exclude_inherited = CORBA::FALSE;



  CORBA_IR::ModuleDef *moduleDef = CORBA_IR::ModuleDef::_narrow(contained);
  if (moduleDef != NULL)
    {
      CORBA_IR::InterfaceName limit_type(desired_interface);
      newContentsResult = moduleDef->contents(limit_type, exclude_inherited);
      update();

      prettyPrintContentsResult();
    }    
  else
    {

      CORBA_IR::InterfaceDef *interfaceDef = CORBA_IR::InterfaceDef::_narrow(contained);      
      if (interfaceDef != NULL)
	{
	  CORBA_IR::InterfaceName limit_type(desired_interface);
	  newContentsResult = interfaceDef->contents(limit_type, exclude_inherited);
          update();
	  prettyPrintContentsResult();
	}

      else
	{
	  cout << "\nSorry, the object you've selected doesn't support contents()";
	}
    }
}




void IRBrowser::lookupNameHelper()
{

    if ((contentsResult == NULL) || (arg >= contentsResult->length())
				 || (arg < 0))
	{
	  cout << "Argument range error.";
	  cout.flush();
	  return;
	}

  
  CORBA_IR::Contained * contained = (*contentsResult)[arg];
  if (contained == NULL)
    {
      cout << "\nError.";
      return;
    }

  char searchName[500];
  cout << "\n search_name? > ";
  cin >> searchName;
  CORBA_IR::Identifier search_name(searchName);

  CORBA::Long levels_to_search;
  cout << "\n levels_to_search? > ";
  cin >> levels_to_search;

  char limitType[500];
  cout << "\n limit_type? > ";
  cin >> limitType;
  CORBA_IR::Identifier limit_type(limitType);  

  CORBA::Boolean exclude_inherited = CORBA::TRUE;
  char exclude_char[10];
  cout << "\n Specify exclude_inherited ? (y) " ;
  cin >> exclude_char;
  if ( exclude_char[0] == 'n' || exclude_char[0] == 'N' )
 	exclude_inherited = CORBA::FALSE;


  CORBA_IR::ModuleDef *moduleDef = CORBA_IR::ModuleDef::_narrow(contained);
  if (moduleDef != NULL)
    {

      newContentsResult = moduleDef->lookup_name(search_name,
					     levels_to_search,
					     limit_type,
					     exclude_inherited);

      update();

      prettyPrintContentsResult();
    }    
  else
    {

      CORBA_IR::InterfaceDef *interfaceDef = CORBA_IR::InterfaceDef::_narrow(contained);      
      if (interfaceDef != NULL)
	{
	  newContentsResult = interfaceDef->lookup_name(search_name,
					     levels_to_search,
					     limit_type,
					     exclude_inherited);
          update();
	  prettyPrintContentsResult();
	}

      else
	{
	  cout << "\nSorry, the object you've selected doesn't support lookup_name()";
	}
    }
}


void IRBrowser::prettyPrintContentsResult()
{
  if (contentsResult == NULL) 
    {
      cout << "\nNULL result";
      return;
    }

  CORBA::Long i;
  cout << "\n";
  for (i = 0; i < contentsResult->length(); i++)
    {
      cout << "# "
	   << i
	   << interfaceType((*contentsResult)[i])
	   << "     "
	   << *((*contentsResult)[i]->name());
      cout << "\n";
    }      	  
}




void IRBrowser::prettyPrintDescriptionResult()
{
  CORBA::Long i;
  cout << "\n";

  if (descriptionResult == NULL)
    {
       cout << "\nNULL result";
       return;
    }

  for (i = 0; i < descriptionResult->length(); i++)
    {
      cout << "# "
	   << i
	   << "    (Container::Description)"
	   << "     "
	   << (*descriptionResult)[i];
      cout << "\n";
    }      	  
}	  


void IRBrowser::lookup_name()
{


  if (arg != -1)
    {
      lookupNameHelper();
      return;
    }      


  char searchName[500];
  cout << "\n search_name? > ";
  cin >> searchName;
  CORBA_IR::Identifier search_name(searchName);

  CORBA::Long levels_to_search;
  cout << "\n levels_to_search? > ";
  cin >> levels_to_search;

  char limitType[500];
  cout << "\n limit_type? > ";
  cin >> limitType;
  CORBA_IR::Identifier limit_type(limitType);  

  CORBA::Boolean exclude_inherited = CORBA::TRUE;
  char exclude_char[10];
  cout << "\n Specify exclude_inherited ? (y) " ;
  cin >> exclude_char;
  if ( exclude_char[0] == 'n' || exclude_char[0] == 'N' )
 	exclude_inherited = CORBA::FALSE;

  newContentsResult = repository->lookup_name(search_name,
					     levels_to_search,
					     limit_type,
					     exclude_inherited);
  update();
  prettyPrintContentsResult();

  return;
}

void IRBrowser::describe_contents()
{
  // If the user has not specified an argument, apply describe_contents() to
  // the Repository itself.  If the user has supplied an argument,
  // _attempt_ to apply describe_contents() to the last computed contentsResult.
  // Since the result of the last computed contentsResult is a sequence
  // of Contained objects (and not Container), a call to this function
  // with an argument will succeed only if we can _narrow the corresponding
  // element in the last contentsResult to something derived from Container,
  // i.e., InterfaceDef or ModuleDef.  We call describeContentsHelper if the user
  // has specified an argument; otherwise this function handles it.

  if (arg != -1)
    {
      describeContentsHelper();
      return;
    }      
 
  cout << "\n Specify limit_type > ";
       
  char desired_interface[500];
  cin >> desired_interface;

  CORBA_IR::InterfaceName limit_type(desired_interface);

  CORBA::Boolean exclude_inherited = CORBA::TRUE;
  char exclude_char[10];
  cout << "\n Specify exclude_inherited ? (y) " ;
  cin >> exclude_char;
  if ( exclude_char[0] == 'n' || exclude_char[0] == 'N' )
 	exclude_inherited = CORBA::FALSE;

  CORBA::Long max_returned_objs = 100;
  
  descriptionResult = repository->describe_contents(limit_type,
						    exclude_inherited,
						    max_returned_objs);

  if (descriptionResult == NULL)
    {
       cout << "\nNULL result";
       return;
    }

  CORBA::Long i;

  cout << "\n";
  for (i = 0; i < descriptionResult->length(); i++)
    {
      cout << "# "
	   << i
	   << "    (Container::Description)"
	   << "     "
	   << (*descriptionResult)[i];
      cout << "\n";
    }  
}


void IRBrowser::describeContentsHelper()
{

    if ((contentsResult == NULL) || (arg >= contentsResult->length())
				 || (arg < 0))
	{
	  cout << "Argument range error.";
	  cout.flush();
	  return;
	}
    
  CORBA_IR::Contained * contained = (*contentsResult)[arg];
  if (contained == NULL)
    {
      cout << "\nError.";
      return;
    }

  cout << "\n Specify limit_type > ";
       
  char desired_interface[500];
  cin >> desired_interface;

  CORBA::Boolean exclude_inherited = CORBA::TRUE;
  char exclude_char[10];
  cout << "\n Specify exclude_inherited ? (y) " ;
  cin >> exclude_char;
  if ( exclude_char[0] == 'n' || exclude_char[0] == 'N' )
 	exclude_inherited = CORBA::FALSE;


  CORBA_IR::ModuleDef *moduleDef = CORBA_IR::ModuleDef::_narrow(contained);
  if (moduleDef != NULL)
    {

      CORBA_IR::InterfaceName limit_type(desired_interface);
      CORBA::Long max_returned_objs = 100;
      descriptionResult
	= moduleDef->describe_contents(limit_type,
				       exclude_inherited,
				       max_returned_objs);

      prettyPrintDescriptionResult();
    }    
  else
    {
      CORBA_IR::InterfaceDef *interfaceDef = CORBA_IR::InterfaceDef::_narrow(contained);      
      if (interfaceDef != NULL)
	{

	  CORBA_IR::InterfaceName limit_type(desired_interface);
	  CORBA::Long max_returned_objs = 100;
	  descriptionResult
	    = interfaceDef->describe_contents(limit_type,
					      exclude_inherited,
					      max_returned_objs);
	  prettyPrintDescriptionResult();
	}

      else
	{
	  cout << "\nSorry, the object you've selected doesn't support contents()";
	}
    }
}


void IRBrowser::describe_interface()
{
    // The user must supply an argument so that we know which Contained
    // object is meant.  Moreover, the object must _narrow to InterfaceDef.

    if (arg == -1)
    {
      cout << "\nYou must supply a #i argument for this operation.";
      return;
    }

    if ((contentsResult == NULL) || (arg >= contentsResult->length())
				 || (arg < 0))
	{
	  cout << "Argument range error.";
	  cout.flush();
	  return;
	}    

    CORBA_IR::Contained * contained = (*contentsResult)[arg];

    CORBA_IR::InterfaceDef *interfaceDef = CORBA_IR::InterfaceDef::_narrow(contained);

    if (interfaceDef == NULL)
    {
      cout << "\nSorry, but the object you've selected isn't an InterfaceDef.";
      return;
    }

    CORBA_IR::InterfaceDef::FullInterfaceDescription* result
      = interfaceDef->describe_interface();

    if (result == NULL)
      {
	cout << "\nInternal error:  FullInterfaceDescription result is NULL?!";
	return;
      }
    
    cout << *result;
  
}


void IRBrowser::base_interfaces()
{
    // The user must supply an argument so that we know which Contained
    // object is meant.  Moreover, the object must _narrow to InterfaceDef.

    if (arg == -1)
    {
      cout << "\nYou must supply a #i argument for this operation.";
      return;
    }

    if ((contentsResult == NULL) || (arg >= contentsResult->length())
				 || (arg < 0))
	{
	  cout << "Argument range error.";
	  cout.flush();
	  return;
	}    

    CORBA_IR::Contained * contained = (*contentsResult)[arg];

    CORBA_IR::InterfaceDef *interfaceDef = CORBA_IR::InterfaceDef::_narrow(contained);

    if (interfaceDef == NULL)
    {
      cout << "\nSorry, but the object you've selected isn't an InterfaceDef.";
      return;
    }

    CORBA_IR::InterfaceDef::RepositoryIds* result
      = interfaceDef->base_interfaces();
    
    if (result == NULL)
      {
	cout << "\nInternal error:  base_interfaces() result is NULL?!";
	return;
      }

    DSResource::ref(result);
    if (result->length() == 0)
      {
	cout << "\nThis interface doesn't inherit from any other interfaces.";
        DSResource::unref(result);
        return;
      }


     releaseContentsResult();
   
     for (CORBA::ULong i=0; i < result->length(); i++) 
    {
       CORBA_IR::RepositoryId element((*result)[i]);

       CORBA_IR::Contained* contained = repository->lookup_id(element);

       contentsResult->append(contained);
    }

    prettyPrintContentsResult();
  
}

void IRBrowser::name()
{
  if (arg == -1) { cout << "\nArgument error"; return; }

    if ((contentsResult == NULL) || (arg >= contentsResult->length())
				 || (arg < 0))
	{
	  cout << "Argument range error.";
	  cout.flush();
	  return;
	}  
  
  cout << *((*contentsResult)[arg]->name());
}

void IRBrowser::id()
{
  if (arg == -1) { cout << "\nArgument error"; return; }
    if ((contentsResult == NULL) || (arg >= contentsResult->length())
				 || (arg < 0))
	{
	  cout << "Argument range error.";
	  cout.flush();
	  return;
	}
  
  cout << *((*contentsResult)[arg]->id());
}

void IRBrowser::defined_in()
{
  if (arg == -1) { cout << "\nArgument error"; return; }

      if ((contentsResult == NULL) || (arg >= contentsResult->length())
				 || (arg < 0))
	{
	  cout << "Argument range error.";
	  cout.flush();
	  return;
	} 
  cout << *((*contentsResult)[arg]->defined_in());
}

void IRBrowser::type()
{
  if (arg == -1) { cout << "\nArgument error"; return; }

// Need to try to narrow() to all classes that supports a typecode
// attribute.
// These classes are AttributeDef, OperationDef (as "result"),
// TypeDef, ConstantDef, and ExceptionDef.

      if ((contentsResult == NULL) || (arg >= contentsResult->length())
				 || (arg < 0))
	{
	  cout << "Argument range error.";
	  cout.flush();
	  return;
	}
  
  CORBA_IR::Contained* contained = (*contentsResult)[arg];
  
  CORBA_IR::AttributeDef * attr = CORBA_IR::AttributeDef::_narrow(contained);

  NCostream strm(cout);

  if (attr == NULL)
   {


     CORBA_IR::OperationDef* op = CORBA_IR::OperationDef::_narrow(contained);

     if (op == NULL)
       {


	 CORBA_IR::ParameterDef* param = CORBA_IR::ParameterDef::_narrow(contained);

	 if (param == NULL)
	   {


	     CORBA_IR::TypeDef* typeDef = CORBA_IR::TypeDef::_narrow(contained);

	     if (typeDef == NULL)
	       {


		 CORBA_IR::ConstantDef* cd = CORBA_IR::ConstantDef::_narrow(contained);

		 if (cd == NULL)
		   {


		     CORBA_IR::ExceptionDef* ed = CORBA_IR::ExceptionDef::_narrow(contained);

		     if (ed == NULL)
		       {
			 cout << "\nSorry, this object doesn't have a type code"
			      << " attribute";
		       }
		     else
		       {
			 CORBA::TypeCode* code = ed->type();
			 if ( code != NULL)
			   code->veryPrettyPrint(strm,0);
			 else cout << "(null)";
		       }
		   }
		 else
		   {
		     CORBA::TypeCode* code = cd->type();
		     if ( code != NULL)
		       code->veryPrettyPrint(strm,0);
		     else cout << "(null)";
		   }
		 
	       }
	     else
	       {

		 CORBA::TypeCode* code = typeDef->type();
		 if ( code != NULL)
		   code->veryPrettyPrint(strm,0);
		 else cout << "(null)";		 
	       }

	   }
	 else
	   {

	     CORBA::TypeCode* code = param->type();
	     if ( code != NULL)
	       code->veryPrettyPrint(strm,0);
	     else cout << "(null)";
	   }
       }
     else
       {

	 CORBA::TypeCode* code = op->result();
	 if ( code != NULL)
	   code->veryPrettyPrint(strm,0);
	 else cout << "(null)";
       }       
   }
  else
   {
     CORBA::TypeCode* code = attr->type();
     if ( code != NULL)
       code->veryPrettyPrint(strm,0);
     else cout << "(null)";
   }  
}


void IRBrowser::value()
{
  if (arg == -1) { cout << "\nArgument error"; return; }

    if ((contentsResult == NULL) || (arg >= contentsResult->length())
				 || (arg < 0))
	{
	  cout << "Argument range error.";
	  cout.flush();
	  return;
	}
  
  CORBA_IR::Contained* contained = (*contentsResult)[arg];
  CORBA_IR::ConstantDef * cd = CORBA_IR::ConstantDef::_narrow(contained);

  if (cd == NULL)
    {
      cout << "\nSorry, value() is applicable only for ConstantDef objects.";
      cout.flush();
      return;
    }

  cout << *(cd->value());
}


void IRBrowser::result()
{
  if (arg == -1) { cout << "\nArgument error"; return; }

// Need to try to narrow() to OperationDef, which is the only
// type of interface that supports the 'result' attribute.

   if ((contentsResult == NULL) || (arg >= contentsResult->length())
				 || (arg < 0))
	{
	  cout << "Argument range error.";
	  cout.flush();
	  return;
	}

  CORBA_IR::Contained* contained = (*contentsResult)[arg];
  
  CORBA_IR::OperationDef * op = CORBA_IR::OperationDef::_narrow(contained);

  if (op == NULL)
    {
       cout << "\nSorry, the 'result' attribute look up applies only to OperationDef.";
       return;
    }

   NCostream strm(cout);
   CORBA::TypeCode* code = op->result();
   if ( code != NULL)
	   code->veryPrettyPrint(strm,0);
   else cout << "(null)";

   return;
    
}

void IRBrowser::mode()
{
  if (arg == -1) { cout << "\nArgument error"; return; }

// Need to try to narrow() to all classes that supports a "mode" attribute.
// These classes are AttributeDef, OperationDef,  and ParameterDef.

      if ((contentsResult == NULL) || (arg >= contentsResult->length())
				 || (arg < 0))
	{
	  cout << "Argument range error.";
	  cout.flush();
	  return;
	}
  
  CORBA_IR::Contained* contained = (*contentsResult)[arg];
  
  CORBA_IR::AttributeDef * attr = CORBA_IR::AttributeDef::_narrow(contained);

  if (attr == NULL)
   {


     CORBA_IR::OperationDef* op = CORBA_IR::OperationDef::_narrow(contained);

     if (op == NULL)
       {

	 CORBA_IR::ParameterDef* param = CORBA_IR::ParameterDef::_narrow(contained);

	 if (param == NULL)
	   {
	     cout << "\nSorry, this object doesn't support mode()";
	   }
	 else
	   {
	     cout << "\n Mode = " << param->mode();
	   }
       }
     else
       {
	 cout << "\n Mode = " << op->mode();  
       }       
   }
  else
   {
     cout << "\n Mode = " << attr->mode();  
   }
}

void IRBrowser::quit()
{
  exit(0);
}

void IRBrowser::show()
{
  prettyPrintContentsResult();
}  


void IRBrowser::info()
{
   getInput();

  while (1)
    {
      if (strcmp(command_buffer,"lookup_id") == 0)
	{
	  cout << "\n[Usage: lookup_id] Used to look up an identifier using a scoped name.";  
	  cout << "\n                   Specify a scoped name when prompted for search_id,";  
	  cout << "\n                   for example, MyModule::MyInterface::myFunction.";  
	  break;
	}
      if (strcmp(command_buffer,"within") == 0)
	{
	  cout << "\n[Usage: #n within] Returns a list of objects that contain current";  
	  cout << "\n                   current object #n.  To see the current object";
          cout << "\n                   list, use the 'show' command.";  
	  break;
	}
      if (strcmp(command_buffer,"describe") == 0)
	{
	  cout << "\n[Usage: #n describe] Used to return the Description";  
	  cout << "\n                     of current object #n.  To see";
	  cout << "\n                     the current object list, use 'show'.";  
	  break;
	}
      if (strcmp(command_buffer,"contents") == 0)
	{
	  cout << "\n[Usage: contents]     Used to return a list of objects";  
	  cout << "\n                      contained by this Repository.";
          cout << "\n";
	  cout << "\n[Usage: #n contents]  Used to return a list of objects";  
	  cout << "\n                      contained by current object #n.  To see";
	  cout << "\n                      the current object list, use 'show'.";  
	  break;
	}
      if (strcmp(command_buffer,"lookup_name") == 0)
	{
	  cout << "\n[Usage: lookup_name] Used to look up an identifier using an unscoped name.";  
	  cout << "\n                     Specify a unscoped name when prompted for search_id,";  
	  cout << "\n                     for example, 'myFunction'. The levels_to_search prompt";  
	  cout << "\n                     expects an integer, see the CORBA spec .  For limit_type,";
	  cout << "\n                     use 'InterfaceDef', 'ModuleDef', etc. or 'all'.";
	  break;
	}
      if (strcmp(command_buffer,"describe_contents") == 0)
	{
	  cout << "\n[Usage: describe_contents]     Used to return a list of Descriptions of objects";  
	  cout << "\n                               contained by this Repository.";
          cout << "\n";
	  cout << "\n[Usage: #n describe_contents]  Used to return a list of Descriptions of objects";  
	  cout << "\n                               contained by current object #n.  To see";
	  cout << "\n                               the current object list, use 'show'.";  
	  break;
	}
      if (strcmp(command_buffer,"describe_interface") == 0)
	{
	  cout << "\n[Usage: #n describe_contents]  Used to print out a FullInterfaceDescription";  
	  cout << "\n                               of current object #n, which is required to be";
	  cout << "\n                               an InterfaceDef.  To see the current object list";
          cout << "\n                               use 'show'.";  
	  break;
	}
      if (strcmp(command_buffer,"base_interfaces") == 0)
	{
	  cout << "\n[Usage: #n base_interfaces]    Used to return a list of base interfaces of";  
	  cout << "\n                               of current object #n, which is required to be";
	  cout << "\n                               an InterfaceDef.  To see the current object list";
          cout << "\n                               use 'show'.";  
	  break;
	}
      if (strcmp(command_buffer,"name") == 0)
	{
	  cout << "\n[Usage: #n name]  Prints name of current object #n.";
	  break;
	}
      if (strcmp(command_buffer,"id") == 0)
	{
	  cout << "\n[Usage: #n id]  Prints of RepositoryId of current object #n.";
	  break;
	}
      if (strcmp(command_buffer,"defined_in") == 0)
	{
	  cout << "\n[Usage: #n id]  Prints of RepositoryId of current object #n definition.";
	  break;
	}
      if (strcmp(command_buffer,"type") == 0)
	{
	  cout << "\n[Usage: #n type]  Prints TypeCode attribute, if any, of current object #n.";
          cout << "\n                  To see current object list, use 'show'";
	  break;
	}
      if (strcmp(command_buffer,"value") == 0)
	{
	  cout << "\n[Usage: #n value]  Prints the 'value' attribute, if any, of current object #n.";
          cout << "\n                   To see current object list, use 'show'";
	  break;
	}
      if (strcmp(command_buffer,"result") == 0)
	{
	  cout << "\n[Usage: #n value]  Prints 'result' attribute, if any, of current object #n.";
          cout << "\n                   To see current object list, use 'show'";
	  break;
	}
      if (strcmp(command_buffer,"mode") == 0)
	{
	  cout << "\n[Usage: #n mode]  Prints 'mode' attribute, if any, of current object #n.";
          cout << "\n                  To see current object list, use 'show'";
	  break;
	}      
      if (strcmp(command_buffer,"quit") == 0)
	{
	  cout << "\n[Usage: quit]  Exits browser";
	  break;
	}
      if (strcmp(command_buffer,"show") == 0)
	{
	  cout << "\n[Usage: show]  Shows list of current objects.  All browser operations with";
	  cout << "\n               a #n argument apply to this implicit list of objects.";
	  break;
	}
      if (strcmp(command_buffer,"info") == 0)
	{
	  cout << "\n[Usage: info command_name]  Prints usage information for command 'command_name'.";
	  break;
	}
      if (strcmp(command_buffer,"help") == 0)
	{
	  cout << "\n[Usage: help]  Prints command summary.";
	  break;
	}
      cout << "\nNo such command name.";
      break;
    }
   
}  

void IRBrowser::help()
{
  printHelp();
}

void IRBrowser::dispatch()
{

// Parses command lines of the form, for example,
//
//     >   quit
//     >   #3 id      (argument must be prefixed with #)
//     >   #2 describe  
//     >   contents
//
//   A number, if present, is assumed to refer to the listing of objects
//   given in the last returned result set, which depends on the context.
//   Additionally, a command such as 'contents' may ask for additional
//   sub-input (not handled here, but always in-line in the implementation
//   of the corresponding function ).  
  
  arg = -1;

  if (command_buffer[0] == '#')
    {
      arg = atoi(command_buffer+1);
      getInput();
    }
  
  while (1)
    {
      if (strcmp(command_buffer,"lookup_id") == 0)
	{
	  lookup_id();
	  break;
	}
      if (strcmp(command_buffer,"within") == 0)
	{
	  within();
	  break;
	}
      if (strcmp(command_buffer,"describe") == 0)
	{
	  describe();
	  break;
	}
      if (strcmp(command_buffer,"contents") == 0)
	{
	  contents();
	  break;
	}
      if (strcmp(command_buffer,"lookup_name") == 0)
	{
	  lookup_name();
	  break;
	}
      if (strcmp(command_buffer,"describe_contents") == 0)
	{
	  describe_contents();
	  break;
	}
      if (strcmp(command_buffer,"describe_interface") == 0)
	{
	  describe_interface();
	  break;
	}
      if (strcmp(command_buffer,"base_interfaces") == 0)
	{
	  base_interfaces();
	  break;
	}
      if (strcmp(command_buffer,"name") == 0)
	{
	  name();
	  break;
	}
      if (strcmp(command_buffer,"id") == 0)
	{
	  id();
	  break;
	}
      if (strcmp(command_buffer,"defined_in") == 0)
	{
	  defined_in();
	  break;
	}
      if (strcmp(command_buffer,"type") == 0)
	{
	  type();
	  break;
	}
      if (strcmp(command_buffer,"value") == 0)
	{
	  value();
	  break;
	}
      if (strcmp(command_buffer,"result") == 0)
	{
	  result();
	  break;
	}
      if (strcmp(command_buffer,"mode") == 0)
	{
	  mode();
	  break;
	}      
      if (strcmp(command_buffer,"quit") == 0)
	{
	  quit();
	  break;
	}
      if (strcmp(command_buffer,"show") == 0)
	{
	  show();
	  break;
	}
      if (strcmp(command_buffer,"info") == 0)
	{
	  info();
	  break;
	}
      if (strcmp(command_buffer,"help") == 0)
	{
	  help();
	  break;
	}
      cout << "\nInvalid command.";
      break;
    }
  printPrompt();
}







	
