//  The client and server are linked into the same program
#include <iostream.h>
#include <unistd.h>
#include <signal.h>
#include "lib_c.hh"
#include "libsrv.h"

void sighandler(int)
{
	exit(0);
}

int main(int argc, char **argv)
{
	signal(SIGINT, sighandler);
	// Create the server instance
	class Library library_server("Harvard");




	CORBA::Boolean 		ret;
	CORBA::Environment env;
	CORBA::Exception	*exp;
	CORBA::SystemException	*system_exp;

	if (argc < 3) {
		cout << "library_client: You must specify an author and title"
			<< endl;
		return 0;
	}
	book book_entry;
	book_entry.author(argv[1]);
	book_entry.title(argv[2]);

	library *library_object = library::_bind(env);
	if (env.check_exception() != 0) {
		exp = env.exception_value();
		system_exp = CORBA::SystemException::_cast(exp);
		if (system_exp != NULL) {
			cout << exp->_id() << " occured: " << endl; 
			cout << "	with minor code " <<
				system_exp->minorFlag() << endl
				<< "	with completion status" <<
				system_exp->completed() << endl;
		}
		library_object->_release();
		return 0;
	}
	else 
		cout << "Bound to library server" << endl;
	
	ret = library_object->add_book(book_entry, env);


	if (env.check_exception() != 0) {
		library::CapacityExceeded *cap_excep;
		exp = env.exception_value();
		cap_excep = library::CapacityExceeded::_cast(exp);

		if (cap_excep == NULL)
			cout << "Exception raised was not CapacityExceeded"
				<< endl;
		else {
			cout <<
			"add_book returned a CapacityExceeded exception "
				<< endl;
			library_object->_release();
			return 0;
		}
	}

	if (ret == CORBA::TRUE)
		cout << "Added book " << argv[1] << ", " << argv[2] 
			<< " to list of books in library." << endl;
	else		
		cout << "Addition of book " << argv[1] << ", " << argv[2] 
			<< " to list failed." << endl;

	library_object->_release();
	return 1;
}


// Server functions

Library::Library(const char *object_name) : library_impl(object_name)
{
}


CORBA::Boolean Library::add_book(const book& book_info, 
				CORBA::Environment& env)
{
	CORBA::Boolean ret;
	if ((ret = bk_list.add_to_list(book_info)) == CORBA::FALSE)
		env.exception_value(new library::CapacityExceeded());
	return ret;
	

}
