#ifndef _bopt_h_
#define _bopt_h_
/****************************************************************************/
/*                                                                          */
/*  ORBeline (c)   is copyrighted by PostModern Computing Technologies, Inc */
/*                                                                          */
/*  Copyright 1993, 1994 by PostModern Computing Technologies, Inc.         */
/*                          1897 Landings Drive                             */
/*                          Mountain View, California, USA                  */
/*                          Telephone (415) 967-6169                        */
/*                          Facsimile (415) 967-6212                        */
/*                                                                          */
/*  This code is the sole property of PostModern Computing Technologies,    */
/*  Inc.  and is  protected  by  copyright under the laws of the United     */
/*  States.   This  program is confidential, proprietary, and  a  trade     */
/*  secret,  not  to  be disclosed  without written authorization  from     */
/*  PostModern Computing Technologies, Inc.  Any use,  duplication,  or     */
/*  disclosure  of  this  program  by other than  PostModern  Computing     */
/*  Technologies, Inc. and its assigned licensees is strictly forbidden     */
/*  by law.                                                                 */
/*                                                                          */
/*  All rights reserved.                                                    */
/*                                                                          */
/*  Author: psm                                                             */
/*                                                                          */
/****************************************************************************/

class ClientEventHandler
{
    public:
	virtual void bind_succeeded(const CORBA::Object *) {}
	virtual void bind_failed(const CORBA::Object *, 
				const CORBA::Exception *) {}
	virtual void server_aborted(const CORBA::Object *) {}
	virtual void rebind_succeeded(const CORBA::Object *obj) {
			bind_succeeded(obj);
	}
	virtual void rebind_failed(const CORBA::Object *obj,
				const CORBA::Exception *exp) {
			bind_failed(obj, exp);
	}
};

class BindOptions
{
    private:
	Boolean			_defer_bind;
	Boolean			_enable_rebind;
	CORBA::ORB::_ORBType	_orb_type_hint;
	ClientEventHandler	*_handler;
	CORBA::Long		_max_bind_tries;
    public:
	BindOptions(Boolean defer_bind= 0, Boolean enable_rebind = 1,
		     CORBA::ORB::_ORBType hint= CORBA::ORB::_ORB_POSTMODERN,
		     ClientEventHandler *handler = NULL,
		     CORBA::Long max_bind_tries = 10) {
		_defer_bind = defer_bind;
		_enable_rebind = enable_rebind;
		_orb_type_hint = hint;
		_handler = handler;
		_max_bind_tries = max_bind_tries;
	}
	BindOptions(const BindOptions& opt) {
		_defer_bind = opt._defer_bind;
		_enable_rebind = opt._enable_rebind;
		_orb_type_hint = opt._orb_type_hint;
		_handler = opt._handler;
		_max_bind_tries = opt._max_bind_tries;
	}
	void operator=(const BindOptions& opt) {
		_defer_bind = opt._defer_bind;
		_enable_rebind = opt._enable_rebind;
		_orb_type_hint = opt._orb_type_hint;
		_handler = opt._handler;
		_max_bind_tries = opt._max_bind_tries;
	}
	virtual ~BindOptions() {}

	Boolean		defer_bind() const { return _defer_bind; }
	Boolean		rebind() const { return _enable_rebind; }
	CORBA::ORB::_ORBType orbType() const { return _orb_type_hint; }
	void 		handler(ClientEventHandler *obj) {
		_handler = obj;
	}
	ClientEventHandler *handler() const { return _handler; }
	void maxRetries(CORBA::Long num) { _max_bind_tries = num; }
	CORBA::Long maxRetries() const { return _max_bind_tries; }
};

#endif
