#ifndef _prop_h_
#define _prop_h_
/****************************************************************************/
/*                                                                          */
/*  ORBeline (c)   is copyrighted by PostModern Computing Technologies, Inc */
/*                                                                          */
/*  Copyright 1993, 1994 by PostModern Computing Technologies, Inc.         */
/*                          1897 Landings Drive                             */
/*                          Mountain View, California, USA                  */
/*                          Telephone (415) 967-6169                        */
/*                          Facsimile (415) 967-6212                        */
/*                                                                          */
/*  This code is the sole property of PostModern Computing Technologies,    */
/*  Inc.  and is  protected  by  copyright under the laws of the United     */
/*  States.   This  program is confidential, proprietary, and  a  trade     */
/*  secret,  not  to  be disclosed  without written authorization  from     */
/*  PostModern Computing Technologies, Inc.  Any use,  duplication,  or     */
/*  disclosure  of  this  program  by other than  PostModern  Computing     */
/*  Technologies, Inc. and its assigned licensees is strictly forbidden     */
/*  by law.                                                                 */
/*                                                                          */
/*  All rights reserved.                                                    */
/*                                                                          */
/*  Author: psm                                                             */
/*                                                                          */
/****************************************************************************/

class Property: public DSResource
{
	DECLARE_NCOBJECT_CLASS(Property);
    private:
	String		_name;
	String		_value;
    public:
	Property(const char *nm, const char *val) : _name(nm), _value(val) {}
	Property() {} 
	Property(const Property& prop) : _name(prop._name), 
				_value(prop._value) {}
	Property& operator=(const Property& prop) {
		_name = prop._name;
		_value = prop._value;
		return *this;
	}
	~Property() {}

	void copyFrom(NCistream& strm);

	const char *name() const { return (const char *)_name; }
	const char *value() const { return (const char *)_value; }

	void name(const char *nm) { _name = nm; }
	void value(const char *val) {_value = val; }
};

class PropertyList: public Sequence
{	
	DECLARE_NCOBJECT_CLASS(PropertyList);
    private:
	Property		**_contents;
	ULong			_count;
	ULong			_num_allocated;
    public:
	static const PropertyList	_nil;

	PropertyList(ULong initial_capacity=16);
	PropertyList(const PropertyList& list);
	PropertyList& operator=(const PropertyList& list);
	~PropertyList();

	void copyFrom(NCistream& strm);

	ULong	length() const { return _count; }
	void 	insert(const Property& elem, ULong index);
	void	insert(Property *elem, ULong index);
	ULong	append(const Property& elem);
	ULong	append(Property *elem);
	void	remove(ULong index);

	Property& operator[](ULong index);
	const Property& operator[](ULong index) const;
	Boolean is_nil() const { return this == &_nil; }
};

#endif

